/*
 * KansiUtilities.java
 *
 * Created on 2008/01/17, 19:10
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

/**
 * 干支を扱うときの補助メソッド。
 * @author 大澤義鷹
 */
public class KansiUtilities {
    
    private KansiUtilities() {
    }
    /**
     * 六十干支の名前からそれが何番目のものかを返す。
     * @return 0 - 59の整数値。
     */
    public static int getKansiValue( String kansi ) {
        if (kansi.length() != 2) {
            throw new IllegalArgumentException("干支名ではない");
        }
        String k = kansi.substring(0,1);
        String s = kansi.substring(1);
        int ik = 0;
        for(String ks : KansiConst.十干) {
            if ( ks.equals(k) ) {
                int is = 0;
                for(String ss : KansiConst.十二支) {
                    if(ss.equals(s)) {
                        return KansiConst.六十干支番号表[ik][is];
                    }
                }
            }
        }
        throw new IllegalArgumentException("干支名ではない");
    }
    /**
     * 干と支の値から、六十干支の何番目のものかを返す。
     */
    public static int getKansiValue( int kan, int si ) {
        if ( kan < 0 || kan >= 10 || si < 0 || kan >= 12) {
            throw new IllegalArgumentException("ありえない干支の組合せ");
        }
        int num = KansiConst.六十干支番号表[kan][si];
        if ( num < 0 ) {
            throw new IllegalArgumentException("ありえない干支の組合せ");
        }
        return num;
    }
}
