/*
 * KyuurekiSupplement.java
 *
 * Created on 2008/01/27, 3:30
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

import java.util.Calendar;

/**
 * 旧暦変換の際の補足情報がこのクラスに書きこまれる。<br>
 * 地方時差による補正値や、23時以降の日付なので翌日に補正したとか。
 * サマータイムの補正が行われたとか。
 * @author 大澤義鷹
 */
public class KyuurekiSupplement {
    /**
     * 経度の指定によってどれだけ時間が補正されたかをミリ秒で表す。
     * マイナスの値になることもある。
     */
    public int adjustTimeByLongitude = 0;
    /**
     * 夏時間の調整が行われた場合にtrueとなる。
     */
    public boolean isDaylightAdjusted = false;
    /**
     * 夏時間の調整が行われたときどれだけ時間が補正されたかをミリ秒で表す。
     */
    public int daylightOffset = 0;
    /**
     * 23時以降の日時が指定されたため、次の日として見なして計算したときはtrue
     * となる。
     */
    public boolean isAdjustedNextDay = false;
    /**
     * KyuurekiAlmanac#getKyuurekiDate()で与えたカレンダーが、諸々の条件で補正され、
     * 補正結果の年月日から旧暦が計算されるのだが、補正されたカレンダーが
     * セットされる。
     */
    public Calendar adjustCalendar = null;
    
    /**  KyuurekiSupplement オブジェクトを作成する */
    public KyuurekiSupplement() {
    }
    /**
     * このオブジェクトの文字列表現を返す。
     * "日の繰上 true,経度時間補正 25分20秒,補正日時 1964-10-01(木)23:00:20,
     * 夏時間実施 false,夏時間補正 00分00秒"
     * といった文字列。
     */
    public String toString() {
        StringBuilder sb = new StringBuilder(120);
        Calendar c = adjustCalendar;
        String date = "";
        if( c != null ) {
            date =String.format("%tF(%ta)%tT",c,c,c);
        }
        sb.append("日の繰上 ");
        sb.append(isAdjustedNextDay);
        sb.append(",経度時間補正 ");
        sb.append( time(adjustTimeByLongitude/1000d/60d) );
        sb.append(",補正日時 ");
        sb.append( date );
        sb.append(",夏時間実施 ");
        sb.append(isDaylightAdjusted);
        sb.append(",夏時間補正 ");
        sb.append( time( daylightOffset / 1000d / 60d) );
        return sb.toString();
    }
    
    private String time(double decimalTime) {
        double [] v = sexagesimals( decimalTime );
        return String.format( "%02d分%02d秒",
                               (int)(v[0]*v[1]), (int)v[2]);
    }
    
    /**
     * 10進小数を60進数表現にして返す。
     * <pre>
     * 戻り値を表示する際の例
     *    System.out.printf("%d %d %f\n",(int)v[1],(int)v[2],v[3]);
     * </pre>
     * @param decimal 十進小数
     * @return 戻り値の[0]は符号で、正の値のときは1.0、負のときは-1.0、ゼロのときは0.0。
     * [1]は度、または時の位。[2]は分、[3]は秒とその端数。
     * [0],[1],[2]はかならず整数が戻る。
     */
    private static double [] sexagesimals(double decimal) {
        double sign = Math.signum(decimal);
        decimal = Math.abs(decimal);
        double th,tm,ts,ws;
        th=(int) decimal;               //整数部抽出→時
        ws = (decimal-th) * 3600d;      //小数部×1時間秒
        ws = ws / 60d;                  //分に換算
        tm = (int)ws;                   //整数部抽出→分
        ws -= tm;                       //小数部抽出
        ts = ws * 60d;                  //×1分秒
        return new double[] { sign,th,tm,ts };
    }
    
//    public static void main(String [] args) {
//        Calendar cal = Calendar.getInstance();
//        KyuurekiSupplement sap = new KyuurekiSupplement();
//        sap.adjustCalendar = cal;
//        System.out.println(sap);
//    }
}
