/*
 * KyuurekiTest4.java
 *
 * Created on 2008/02/21, 20:02
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import junit.framework.TestCase;

/**
 * 旧暦データファイルを読みこんで、Kyuureki#getKyuurekiに与えて、
 * すべてのデータが正しく旧暦に変換できるか検査する。
 * 実行してエラーメッセージが出なければ正常。
 * これはファイルを読みながら検査するので、正確なベンチとはいえない。
 * また暦データのCSVファイルは複数に分割されているのに、そのうちの一つしか検査
 * していない。
 * 正確なテストにはKyuurekiTest4を使用したほうが良い。
 * ただこのプログラムは非常にシンプルなので、良さもあると思い残してある。
 * @author 大澤義鷹
 */
public class KyuurekiTest1 extends TestCase {
    static final int DATE = 0;
    static final int NEWMOON = 5;
    static final int MONTH = 6;
    static final int DAY = 7;
    static final int LEAP = 8;
    
    /**  KyuurekiTest4 オブジェクトを作成する */
    public KyuurekiTest1(String testName ) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }
    protected void tearDown() throws Exception {
    }
    
    public String getString(boolean leap,int month,int day) {
        String ls = ( leap ) ? "閏" : "";
        return String.format("%s%02d月%02d日",ls,month,day);
    }
        
    public void test() throws Exception {
        File file = new File("c:/src/perl/kyuureki2/kyuureki1920.csv");
        KyuurekiAlmanac kyuureki = new KyuurekiAlmanac();
        BufferedReader reader = null;
        reader = new BufferedReader(new FileReader(file));
        String [] cols = new String[10];
        int j = 0;
        long t = System.currentTimeMillis();
        while ( reader.ready() ) {
            String line = reader.readLine();
            String [] temp = line.split(",");
            for(int i=0; i<cols.length; i++) cols[i] = "";
            for(int i=0; i<temp.length; i++) cols[i] = temp[i];
            int ymd = Integer.parseInt( cols[ DATE ] );
            int y = ymd / 10000;
            int m = (ymd - ( y * 10000 )) / 100;
            int d = ymd % 100;
            int kmonth = Integer.parseInt( cols[MONTH] );
            int kday   = Integer.parseInt( cols[ DAY ] );
            boolean leap = cols[ LEAP ].equals("leap");
            String original = getString(leap,kmonth,kday);
            if ( ymd >= kyuureki.getMaximumRange() ) break;
            KyuurekiDate kdate = kyuureki.getKyuurekiDate(y,m,d);
            assertEquals( kmonth, kdate.month );
            assertEquals( kday,   kdate.day );
            assertEquals( leap,   kdate.leap );
//            if ( kmonth != kdate.month || kday != kdate.day || leap != kdate.leap ) {
//                System.out.println("ERROR : " + ymd + " " + kdate);
//            }
            j++;
        }
        System.out.println("処理件数 " + j + "件");
        System.out.println((System.currentTimeMillis() - t)/1000d + "[sec]");
        reader.close();        
    }
}
