/*
 * WindowHandler.java
 *
 * Created on 2008/02/12, 1:41
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.michiteru;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Properties;

/**
 * フレームの移動やリサイズの際に、それらのパラメターをプロパティに書きこむ。
 * このクラスのインスタンスは親となるFrameのComponentListenerとして登録して使う。
 * @author 大澤義孝
 */
class WindowHandler extends ComponentAdapter {
    Frame component;
    Properties masterProp;
    WindowHandler(Frame owner,Properties masterProp) {
        this.component = owner;
        this.masterProp = masterProp;
    }
    void setRect(ComponentEvent e) {
        Rectangle rect = e.getComponent().getBounds();
        masterProp.setProperty( "frame.rect", String.format("%d,%d,%d,%d",
            rect.x,rect.y,rect.width,rect.height));
        component.setPreferredSize(rect.getSize());
    }
    @Override
    public void componentMoved(ComponentEvent e) {
        setRect(e);
    }
    @Override
    public void componentResized(ComponentEvent e) {
        setRect(e);
    }
    public void setBounds() {
        String value = masterProp.getProperty("frame.rect");
        if ( value == null ) {
            //component.setPreferredSize(new Dimension(600,600));
            component.setLocationRelativeTo(null);
        } else {
            String [] v = value.split(",");
            Rectangle r = new Rectangle( Integer.parseInt(v[0]),
                Integer.parseInt(v[1]),
                Integer.parseInt(v[2]),
                Integer.parseInt(v[3]) );
            component.setBounds(r);
            component.setPreferredSize(r.getSize());
        }
    }
}
