/*
 * MethodTable.java
 *
 * Created on 2008/01/15, 22:31
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.chartlib;

import java.awt.Color;
import to.tetramorph.starbase.chartparts.GBoxLayout;
import to.tetramorph.starbase.chartparts.GLabel;
import to.tetramorph.starbase.chartparts.ZodiacPanel;

/**
 * ZodiacPanel上にハウス分割法や進行法を表示する部品。
 * @author 大澤義鷹
 */
public class MethodTable extends GBoxLayout {
    private static final int G_CENTER = GLabel.CENTER;
    private static final int G_RIGHT  = GLabel.RIGHT;
    private static final int G_BOTTOM = GLabel.BOTTOM;
    private static final int G_LEFT = GLabel.LEFT;
    private static final int G_TOP = GLabel.TOP;

    private GLabel houseMethodLabel = new GLabel("ハウス分割法：",G_RIGHT );
    private GLabel progressMethodLabel = new GLabel("進行法：",G_RIGHT);
    private GLabel houseMethodNameLabel = new GLabel();
    private GLabel progMethodNameLabel = new GLabel();

    /**
     * MethodTable オブジェクトを作成する。
     */
    public MethodTable() {
        super( GBoxLayout.HORIZONTAL );
    }
    
    /**
     * ハウス分割法や進行法を表示するためのレイアウトを作成。
     */
    public void updateMethodTable( int showRing ) {
        int r = showRing;
        this.removeAll();
        if(r == 1 || r == 3 || r == 4) { //P円がないときは進行法は非表示
            houseMethodNameLabel.setInset( G_BOTTOM, 0 );
            add( houseMethodLabel, houseMethodNameLabel);
        } else {
            GBoxLayout lay1 = new GBoxLayout( GBoxLayout.VERTICAL );
            houseMethodNameLabel.setInset( G_BOTTOM, 0.5 );
            houseMethodLabel.setInset( G_BOTTOM, 0.5 );
            lay1.add( houseMethodLabel, progressMethodLabel );
            GBoxLayout lay2 = new GBoxLayout(GBoxLayout.VERTICAL);
            lay2.add( houseMethodNameLabel, progMethodNameLabel);
            add( lay1, lay2 );
        }
        //setLocation( 0, 70, G_LEFT, G_BOTTOM);
    }
    

    /**
     * 色をセットする。
     */
    public void setColor( Color listOtherColor ) {
        houseMethodNameLabel.setColor( listOtherColor );
        houseMethodLabel.setColor( listOtherColor );
        progMethodNameLabel.setColor( listOtherColor );
        progressMethodLabel.setColor( listOtherColor );        
    }
    
    /**
     * ハウス分割法と進行法の名前をセットする。
     */
    public void setMethodNames(String hmeth,String pmeth) {
        houseMethodNameLabel.setText( hmeth );
        progMethodNameLabel.setText( pmeth );
    }
}
