/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.dict;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import javax.swing.MutableComboBoxModel;

/**
 * 辞書にアクセスするインターフェイス。
 * 「アクション名」とはプラグイン側が用意するxmlファイルの中の&lt;action>
 * タグのnameエレメントの値。
 * @author 大澤義鷹
 */
interface Book {

    DictAction getAction(String action);

    /**
     * アクション選択のコンボボックスモデルを返す。
     * @return
     */
    MutableComboBoxModel getActionComboBoxModel();

    /**
     * 使用できるアクション名のリストを返す。
     * @return
     */
    List<String> getActionList();

    /**
     * 指定されたアクション名の各パラメターの値を自然言語に変換して返す。
     * @param action アクション名
     * @param key 変換したいキー。
     * @return 自然言語に変換されたキー。
     */
    String getActionTitle(String action, String key);

    File getFile();
    DictNode getRootNode();
    /**
     * この本のXML表現をバイト配列で返す。
     * @param p "editable=treu/false"の値のみ認識する。
     * @return 本の内容を表現したXMLのデータをバイト配列で返す。構文エラー等が
     * あった場合はnullを返す。
     */
    //byte [] getByteArray( Properties p);
    public Map<String,ActionTranslator> getTransMap();
    public Properties getBookProperties();
    /**
     * 本のタイトルを返す。タイトルとはノードのトップのUserObjectをStringとみな
     * したもの。
     * @return
     */
    public String getTitle();
    /**
     * この本が編集可能な場合はtrueを返す。編集禁止ならfalseを返す。
     * @return
     */
    public boolean isEditable();
    public String getDictType();
}
