/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.dict;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

/**
 * プラグイン側から提供される送信アクションのデータを一個分格納するクラス。
 * @author 大澤義鷹
 */
public class DictAction {
    private Map<String,Set<String>> paramMap = new LinkedHashMap<String,Set<String>>();
    private String actionName;
    /**
     * オブジェクトを作成する。
     * @param actionName "SelectedBody"等、アクション名を指定する。
     */
    public DictAction( String actionName ) {
        this.actionName = actionName;
    }
    /**
     * 指定されたフィールドキーとその値をこのオブジェクトに登録する。
     * @param key "planet","sign"等を指定する。
     * @param values "sun,mon,mer"などカンマで区切られた文字列を指定。
     */
    public void setParam( String key, String values ) {
        Set<String> vset = new LinkedHashSet<String>();
        for ( String value : values.split(",") )
            vset.add( value );
        paramMap.put( key, vset );
    }

    /**
     * 指定されたフィールドキーに対する値の列挙を返す。
     * @param key "planet"を指定すると、"sun","mon",,,等。
     */
    public Iterator<String> paramIterator( String key ) {
        return paramMap.get(key).iterator();
    }

    /**
     * このオブジェクトのアクション名を返す。
     */
    public String getActionName() {
        return actionName;
    }

    /**
     * このオブジェクトにアクション名をセットする。
     */
    public void setActionName(String actionName) {
        this.actionName = actionName;
    }
    /**
     * このオブジェクトに登録されているパラメターの数を返す。
     */
    public int getParamCount() {
        return paramMap.size();
    }
    /**
     * パラメターのキー列挙を返す。
     */
    public Iterator<String> keyIterator() {
        return paramMap.keySet().iterator();
    }
}
