/*
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.starbase.formatter;

import javax.swing.JFormattedTextField;

/**
 * 日付や時刻や緯度経度等を入力するTextFieldにセットするフォーマッタは
 * このクラスを継承して作成する。
 * それによって全角→半角への変換メソッドを共有できる。
 */
public abstract class AbstractFormatter 
                                extends JFormattedTextField.AbstractFormatter {
    /**
     * 文字列中の全角数字を半角に変換
     * ANK → Alphabet Numeric Kana
     */
    protected static String zenkakuToANK(String value) {
        char [] buf = value.toCharArray();
        StringBuilder sb = new StringBuilder();
        for ( int i=0; i < buf.length; i++ ) {
            if ( buf[i] >= '０' && buf[i] <= '９' ) {
                sb.append( (char)( buf[i] - '０' + '0' ) );
            } else if ( buf[i] >= 'Ａ' && buf[i] <= 'Ｚ' ) {
                sb.append( (char)( buf[i] - 'Ａ' + 'A' ) );
            } else if ( buf[i] >= 'ａ' && buf[i] <= 'ｚ' ) {
                sb.append((char)(buf[i] - 'ａ' + 'a'));
            } else if ( buf[i] == '　' ) {
                sb.append(' ');
            } else if ( buf[i] == '．' ) {
                sb.append('.'); 
            } else if ( buf[i] == '，' || buf[i] == '、' || buf[i] == '。' ) {
                sb.append(',');
            } else sb.append( buf[i] );
        }
        return sb.toString();
    }
    
    /**
     * テキストの中に括弧でかこまれた部分を検出したら除去する。
     * 1999年(平成○年)6月2日なら、1999年6月2日。
     * 1999年6月2日(木曜日)なら、1999年6月2日。
     * ただし完全なアルゴリズムではない。二重に括弧が続くとダメ(hoge(payo))など。
     */
    protected static String 括弧除去( String text ) {
        StringBuilder sb = new StringBuilder( text.length() );
        boolean mask = false;
        for ( int i=0; i < text.length(); i++ ) {
            String c = text.substring(i,i+1);
            if ( c.matches("[\\(（［\\[]") ) mask = true;
            if ( ! mask ) sb.append(c);
            if ( c.matches("[\\)）］\\]]")) mask = false;
        }
        return sb.toString();
    }
    
}
