/*
 * NodePreferencePanel.java
 *
 * Created on 2006/09/15, 7:07
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.starbase;

import javax.swing.ButtonGroup;
import to.tetramorph.util.Preference;

/**
 * ムーンノードとアポジー(リリス)に関する設定パネル。
 * regist()が呼ばれたタイミングでプレファランスに次のプロパティを設定する。
 * <pre>
 * UseMeanNode = true/false
 *  trueならミーンノードを使用。falseならトルーノードを使用。
 * UseMeanApogee = true/false
 *  trueならミーンアポジーを使用。falseならオスカレーションアポジーを使用。
 * </pre>
 */
class NodePreferencePanel extends PreferencePanel {

    ButtonGroup nodeButtonGroup = new ButtonGroup();
    ButtonGroup apogeeButtonGroup = new ButtonGroup();

    /** Creates new form NodePreferencePanel */
    public NodePreferencePanel() {
        initComponents();
        init();
    }

    private void init() {
        nodeButtonGroup.add(nodeRadioButton1);
        nodeButtonGroup.add(nodeRadioButton2);
        apogeeButtonGroup.add(apogeeRadioButton1);
        apogeeButtonGroup.add(apogeeRadioButton2);
        if ( Config.usr.getBoolean("UseMeanNode", false)) {
            nodeRadioButton2.doClick();
        } else {
            nodeRadioButton1.doClick();
        }
        if ( Config.usr.getBoolean("UseMeanApogee", false)) {
            apogeeRadioButton2.doClick();
        } else {
            apogeeRadioButton1.doClick();
        }
    }

    @Override
    public void regist() {
        String v1 = nodeButtonGroup.getSelection().getActionCommand();
        Config.usr.setProperty("UseMeanNode", v1);

        String v2 = apogeeButtonGroup.getSelection().getActionCommand();
        Config.usr.setProperty("UseMeanApogee", v2);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
  // <editor-fold defaultstate="collapsed" desc=" Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {
    java.awt.GridBagConstraints gridBagConstraints;
    javax.swing.JLabel jLabel1;
    javax.swing.JLabel jLabel2;
    javax.swing.JLabel jLabel3;
    javax.swing.JLabel jLabel4;

    jLabel1 = new javax.swing.JLabel();
    jLabel2 = new javax.swing.JLabel();
    nodeRadioButton1 = new javax.swing.JRadioButton();
    nodeRadioButton2 = new javax.swing.JRadioButton();
    jLabel3 = new javax.swing.JLabel();
    apogeeRadioButton1 = new javax.swing.JRadioButton();
    apogeeRadioButton2 = new javax.swing.JRadioButton();
    jLabel4 = new javax.swing.JLabel();

    setLayout(new java.awt.GridBagLayout());

    jLabel1.setText("\u30e0\u30fc\u30f3\u30ce\u30fc\u30c9\u306e\u8a2d\u5b9a");
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridwidth = 2;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
    add(jLabel1, gridBagConstraints);

    jLabel2.setText("\u3000\u3000");
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 1;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
    add(jLabel2, gridBagConstraints);

    nodeRadioButton1.setText("\u771f\u30ce\u30fc\u30c9");
    nodeRadioButton1.setActionCommand("false");
    nodeRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
    nodeRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 1;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
    add(nodeRadioButton1, gridBagConstraints);

    nodeRadioButton2.setText("\u5e73\u5747\u30ce\u30fc\u30c9");
    nodeRadioButton2.setActionCommand("true");
    nodeRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
    nodeRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 2;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
    add(nodeRadioButton2, gridBagConstraints);

    jLabel3.setText("\u6708\u306e\u9060\u5730\u70b9\u306e\u8a2d\u5b9a");
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 4;
    gridBagConstraints.gridwidth = 2;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
    add(jLabel3, gridBagConstraints);

    apogeeRadioButton1.setText("\u5bc6\u63a5\u30ea\u30ea\u30b9");
    apogeeRadioButton1.setActionCommand("false");
    apogeeRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
    apogeeRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 5;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
    add(apogeeRadioButton1, gridBagConstraints);

    apogeeRadioButton2.setText("\u5e73\u5747\u30ea\u30ea\u30b9");
    apogeeRadioButton2.setActionCommand("true");
    apogeeRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
    apogeeRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 6;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
    add(apogeeRadioButton2, gridBagConstraints);

    jLabel4.setText("\u3000");
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 3;
    add(jLabel4, gridBagConstraints);

  }// </editor-fold>//GEN-END:initComponents
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JRadioButton apogeeRadioButton1;
  private javax.swing.JRadioButton apogeeRadioButton2;
  private javax.swing.JRadioButton nodeRadioButton1;
  private javax.swing.JRadioButton nodeRadioButton2;
  // End of variables declaration//GEN-END:variables
}
