/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

/*
 * StopWatch.java
 *
 * Created on 2009/11/16, 3:29:30
 */

package to.tetramorph.util;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * 時間計測に使う簡易ツール。
 * set()してshow()されるまでの経過時間を標準出力にエコーする。
 * <pre>
 * 使い方
 * StopWatch.set("起動までの時間");
 * .........様々な処理..........
 * StopWatch.show("起動までの時間");
 * (コンソール出力例)
 * 起動までの時間 = 17.05 [sec]
 * </pre>
 * @author 大澤義孝
 */
public class StopWatch {
    static Map<String,Long> map = new HashMap<String,Long>();
    /**
     * 時間計測を開始する。
     * @param key 適当な計測時間名を与える。
     */
    public static void set( String key ) {
        map.put( key, System.currentTimeMillis() );
    }

    /**
     * セットされてからの時間を標準出力にエコーする。
     * @param key set()で指定した計測時間名前。
     * "起動時間 = 20.00[s]"などと表示される。
     */
    public static void show(String key) {
        double dt = (System.currentTimeMillis()- map.get(key)) / 1000.;
        Logger.getLogger( StopWatch.class.getName() )
                .log( Level.INFO, String.format("%s = %4.2f [sec]",key, dt) );
    }
    /**
     * showは経過時間をログ(通常、標準出力)に表示するが、このメソッドはその値を
     * 返す。ただし、秒以下の値は切り捨てられる。
     * @param key
     * @return
     */
    public static long getTimeBySec( String key ) {
        return (long)((System.currentTimeMillis()- map.get(key)) / 1000.);
    }
}
