/*
 * TableColumnResizer.java
 *
 * Created on 2006/08/03, 6:27
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.util;

import java.awt.Component;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * JTableの各列をセル内の文字列長に応じて適切なサイズにリサイズするツール。
 */
public class TableColumnResizer {
    private TableColumnResizer() { }
    /**
     * JTableの各列を適切なサイズにリサイズする。このクラスはstaticメソッドのadjust
     * を呼び出せば使うことができるが、イベントキューから呼び出さなければならない。
     * たとえば次のようにして呼び出す
     * <pre>
     * SwingUtilities.invokeLater(new Runnable() {
     *  public void run() {
     *    TableColumnResizer.adjust( jTable );
     *    revalidate();
     *  }
     * });
     * </pre>
     * @param table 列幅を調節したいJTableオブジェクト
     */
    public static void adjust(JTable table) {
        TableColumnModel columnModel = table.getColumnModel();
        for(int col=0; col<table.getColumnCount(); col++) {
            int maxwidth= 0;
            for(int row=0; row<table.getRowCount(); row++) {
                TableCellRenderer rend = table.getCellRenderer(row,col);
                Object value = table.getValueAt(row,col);
                Component comp = rend.getTableCellRendererComponent(
                    table, value, false, false, row,col );
                maxwidth = Math.max(comp.getPreferredSize().width,maxwidth);
            }
            TableColumn column = columnModel.getColumn(col);
            column.setPreferredWidth(maxwidth);
        }
    }
}
