/*
 * ResizeLabel.java
 *
 * Created on 2008/01/01, 8:30
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.widget;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import javax.swing.Icon;
import javax.swing.JLabel;

/**
 * ウインドウの右隅に表示する、リサイズ可能であることを示すアイコンラベル。
 * このラベルをダイアログやフレームみ右下のコーナーに配置する方法の定石は、
 * BorderLayoutをセットした2枚のJPanelを用意して、東の位置に2枚目をセット。
 * 2枚目のパネルの南にResizeLabelをセットする。
 * <pre>
 * +--------+-------+
 * |        |E      |    E = East
 * |        |+-----+|
 * |        |+-----+|
 * |        ||S   *||    S = South
 * |        |+-----+|    * = ResizeLabel
 * +--------+-------+
 * </pre>
 * それでこのような入れ子を作るのが面倒なときは、ResizeIconPanelを使用する。
 * @see ResizeIconPanel
 * @author 大澤義鷹
 */
public class ResizeLabel extends JLabel {
    static final Color darkColor = new Color(122,138,153);
    static final Color lightColor = new Color(255,255,255);
    static final int [][] tile = {
        { 0, 0, 1 },
        { 0, 1, 1 },
        { 1, 1, 1 },
    };

    /**  ResizeLabel オブジェクトを作成する */
    public ResizeLabel() {
        super("");
        setIcon( new CornerIcon() );
        setOpaque(true);
        setText("");
    }

    class CornerIcon implements Icon {
        @Override
        public void paintIcon(Component c, Graphics g, int x, int y) {
            Color oldColor = g.getColor();
            for ( int v=0; v<tile.length; v++ ) {
                for ( int u=0; u<tile[v].length; u++ ) {
                    if ( tile[v][u] == 0 ) continue;
                    int xp = u * 4 + 1 + x;
                    int yp = v * 4 + 1 + y;
                    g.setColor(lightColor);
                    g.fillRect( xp+1, yp+1, 2, 2 );
                    g.setColor( darkColor );
                    g.fillRect( xp, yp, 2, 2 );
                }
            }
            g.setColor(oldColor);
        }

        @Override
        public int getIconWidth() {
            return 12;
        }

        @Override
        public int getIconHeight() {
            return 12;
        }
    }
}
