/*
 * IntegerDate.java
 *
 * Created on 2008/02/19, 17:55
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.time;

import java.util.GregorianCalendar;
import java.util.TimeZone;
import static java.util.Calendar.*;
import java.util.Calendar;
/**
 * 日付を"20080219"(2008年2月19日を表現)などと整数で表現して操作するための
 * メソッド。
 * @author 大澤義鷹
 */
public class IntegerDate {
    
    private IntegerDate() {
    }
    
    /**
     * ユリウス日をグレゴリオ暦に変換してyyyymmdd形式のint型で返す。
     * 紀元前の場合はマイナスの値を返す。
     * @param jday ユリウス日
     * @param tz   タイムゾーン
     * @return 20080219といった整数表現の日付。
     */
    public static int revjday( double jday,TimeZone tz) {
        GregorianCalendar c = JDay.getCalendar(jday,tz);
        int date =   c.get( YEAR ) * 10000 
                 + ( c.get( MONTH ) + 1 ) * 100
                 +   c.get( DAY_OF_MONTH );
        if ( c.get( Calendar.ERA ) == GregorianCalendar.BC ) {
            date = -date;
        }
        return date;
    }
    /**
     * yyyymmddの形式の日付データを年月日に分解してymdに書きこむ。
     * [0]=年,[1]=月,[2]=日。
     */
    public static void reverse( int date, int [] ymd) {
        ymd[0] = date / 10000;
        ymd[1] = (date - ( ymd[0] * 10000 )) / 100;
        ymd[2] = date % 100;        
    }
    /**
     * 年月日をyyyymmdd形式のint型データに変換して返す。
     */
    public static int get(int year,int month,int day) {
        return year * 10000 + month * 100 + day;
    }
    
}
